import click
from ecsclient.common.exceptions import ECSClientException

from .._util.exceptions import EcsmgmtClickException
from .._util.format import success_echo, pretty_table


@click.command()
@click.argument('user-id')
@click.option('-n', '--namespace', type=click.STRING, show_default=True, help='Namespace of the user')
@click.option('-e', '--expiry-time', type=click.INT, help='Expiry time in minutes for the previous secret key.')
@click.option('-k', '--secret-key', type=click.STRING, help='Provide secret key instead of generating one.')
@click.pass_obj
def cli(obj, user_id, namespace, expiry_time, secret_key):
    """Create new secret key
    """
    client = obj['client']

    try:
        res = client.secret_key.create(user_id=user_id, namespace=namespace, expiry_time=expiry_time,
                                       secret_key=secret_key)
        data = [
            ('Secret Key:', res['secret_key']),
        ]
        msg = pretty_table(data, tablefmt='plain')
        if expiry_time is not None:
            res = client.secret_key.get(user_id=user_id, namespace=namespace)
            msg += f'\nOld secret key "{res["secret_key_1"]}" expires at "{res["key_expiry_timestamp_1"]}".'
        success_echo(f'Created secret key for user "{user_id}" in namespace "{namespace}":\n{msg}')
    except ECSClientException as e:
        raise EcsmgmtClickException(e.message)



import os.path

import yaml


def load_config(ctx, param, config_path):
    configpath = os.path.abspath(os.path.expanduser(config_path))

    try:
        with open(configpath, 'r') as infile:
            config = yaml.safe_load(infile.read())
    except FileNotFoundError:
        return

    try:
        options = dict(config['defaults'])
    except KeyError:
        options = {}

    try:
        namespace_functions = {
            'user': ['get', 'create', 'delete'],
            'secret-key': ['get', 'create', 'delete'],
        }
        namespace_options = {group: {command: {'namespace': options['namespace']} for command in commands} for
                             group, commands in namespace_functions.items()}
        options.update(namespace_options)
    except KeyError:
        pass

    ctx.default_map = options
    ctx.obj = {
        'config': config,
    }



import click
from ecsclient.common.exceptions import ECSClientException

from .._util.exceptions import EcsmgmtClickException
from .._util.format import success_echo


@click.command()
@click.argument('user-id')
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, user_id, namespace):
    """Create new object user
    """
    client = obj['client']

    try:
        client.object_user.delete(user_id=user_id, namespace=namespace)
        success_echo(f'deleted user "{user_id}" in namespace "{namespace}"')
    except ECSClientException as e:
        raise EcsmgmtClickException(e.message)



import click
from ecsclient.common.exceptions import ECSClientException

import inquirer

from .._util.format import success_echo
from .._util.exceptions import EcsmgmtClickException


@click.command()
@click.argument('user-id')
@click.option('-n', '--namespace', type=click.STRING, show_default=True, help='Namespace of the user')
@click.option('-k', '--secret-key', type=click.STRING, help='Secret key which should get deleted')
@click.pass_obj
def cli(obj, user_id, namespace, secret_key):
    """Deletes secret key
    """
    client = obj['client']

    res = client.secret_key.get(user_id=user_id, namespace=namespace)
    existing_secret_keys = []
    for i in [1, 2]:
        if res[f'secret_key_{i}_exist']:
            existing_secret_keys.append(res[f'secret_key_{i}'])

    # abort if no keys are present
    if not existing_secret_keys:
        raise EcsmgmtClickException(f'User "{user_id}" in namespace "{namespace}" has no secret keys set.')

    if len(existing_secret_keys) > 1 and secret_key is None:
        secret_key_options = [
            inquirer.Checkbox(
                'secret_keys_to_delete',
                message=f'User "{user_id}" in namespace "{namespace}" has both secret keys set, which should be deleted?',
                choices=existing_secret_keys,
            )
        ]
        answers = inquirer.prompt(secret_key_options)
        secret_keys_to_delete = answers['secret_keys_to_delete']

        if len(secret_keys_to_delete) == 0:
            click.echo("Nothing was deleted!")
            return
    # abort if given secret_key is not assigned to user
    elif secret_key is not None and secret_key not in existing_secret_keys:
        raise EcsmgmtClickException(f'User "{user_id}" in namespace "{namespace}" has no secret key "{secret_key}".')
    else:
        secret_keys_to_delete = existing_secret_keys

    try:
        if len(secret_keys_to_delete) == 2:
            client.secret_key.delete(user_id=user_id, namespace=namespace, secret_key=None)
        else:
            for key in secret_keys_to_delete:
                client.secret_key.delete(user_id=user_id, namespace=namespace, secret_key=key)
        key_list = ", ".join([f'"{item}"' for item in secret_keys_to_delete])
        success_echo(f'Deleted secret-keys {key_list} from user "{user_id}" in namespace "{namespace}"')
    except ECSClientException as e:
        raise EcsmgmtClickException(e.message)



import click

from .._util.core import DynamicMultiCommandFactory

DynamicMultiCommand = DynamicMultiCommandFactory().create(__file__, __package__)


@click.command(cls=DynamicMultiCommand, name='user')
def cli():
    """Manage object users
    """



import click
from ecsclient.common.exceptions import ECSClientException

from .._util.exceptions import EcsmgmtClickException
from .._util.format import success_echo


@click.command()
@click.argument('user-id', type=click.STRING)
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.option(
    '-t', '--tag', 'tags',
    multiple=True,
    type=click.STRING,
    help='User tags seperated by ",", option can be provided multiple times'
)
@click.pass_obj
def cli(obj, user_id, namespace, tags):
    """Create new object user
    """
    client = obj['client']

    sane_tags = []
    for tag in tags:
        splited = tag.split(',')
        sane_tags.extend(splited)

    try:
        client.object_user.create(user_id=user_id, namespace=namespace, tags=sane_tags)
        success_echo(f'created user "{user_id}" in namespace "{namespace}"')
    except ECSClientException as e:
        raise EcsmgmtClickException(e.message)



import click
from tabulate import tabulate


def pretty_table(data: list, headers: list = None, tablefmt: str = 'github') -> str:
    """Tabulate wrapper function with predefined format"""
    if headers is None:
        table = tabulate(data, tablefmt=tablefmt)
    else:
        table = tabulate(data, headers=headers, tablefmt=tablefmt)
    return table


def success_echo(message: str):
    click.secho('Success: ', nl=False, fg='green', bold=True)
    click.echo(message)



import typing as t
from gettext import gettext as _

from click import ClickException, echo, secho
from click._compat import get_text_stderr


class EcsmgmtClickException(ClickException):
    def show(self, file: t.Optional[t.IO] = None) -> None:
        if file is None:
            file = get_text_stderr()

        secho(_("Error: "), nl=False, fg='red', bold=True)
        echo(_("{message}").format(message=self.format_message()), file=file)



import click

from .._util.format import pretty_table


@click.command(name='list')
@click.pass_obj
def cli(obj):
    """List existing users
    """
    client = obj['client']

    user_request = client.object_user.list()
    user_list = [list(entry.values()) for entry in user_request['blobuser']]
    headers = ['Namespace', 'User ID']
    table = pretty_table(user_list, headers)
    click.echo(table)



"""Implements enhancments for click core utilities"""

import importlib
import os

import click


class DynamicMultiCommandFactory:
    """A factory to make configurable dynamic click.MultiCommand

    The main purpose of the factory is the configuration of the dynamic click.MultiCommand.
    It is a quirk, but currently there is no better implentation for this purpose.
    Feel free to find a better solution.
    """

    def create(self, parent_path, base_package):
        """Create a DynamicMultiCommand and uses parent_path as search path.

        Keyword arguments:
        parent_path -- search parent_path for sub commands
        base_package -- base package from where the subcommands are imported
        """

        # strip parent filename from path
        plugin_path = os.path.dirname(parent_path)

        class DynamicMultiCommand(click.MultiCommand):
            """A dynamic Multicommand implementation of click.MultiCommand

            Commands are dynamically """

            def list_commands(self, ctx):
                commands = []
                for pathname in os.listdir(plugin_path):
                    if pathname.startswith('_'):
                        continue

                    if pathname.endswith('.py'):
                        commands.append(pathname[:-3])
                    elif os.path.isdir(os.path.join(plugin_path, pathname)):
                        commands.append(pathname)

                commands.sort()
                return commands

            def get_command(self, ctx, cmd_name):
                commands = self.list_commands(ctx)
                command_candidates = [c for c in commands if c.startswith(cmd_name)]

                if len(command_candidates) == 1:
                    command = command_candidates[0]
                elif len(command_candidates) > 1:
                    ctx.fail("Too many command matches: {matches}".format(
                        matches=', '.join(sorted(command_candidates))))
                else:
                    command = None

                if command is None:
                    return_command = None
                else:
                    try:
                        command_module = importlib.import_module(".{name}".format(name=command), base_package)
                        return_command = command_module.cli
                    except ModuleNotFoundError:
                        return_command = None

                return return_command

        return DynamicMultiCommand



import click

from .._util.format import pretty_table


@click.command(name='get')
@click.argument('user-id', type=click.STRING)
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, user_id, namespace):
    """Get user details
    """
    client = obj['client']

    res = client.object_user.get(user_id=user_id, namespace=namespace)
    lines = [(key, val) for key, val in res.items()]
    msg = pretty_table(lines, tablefmt='plain')
    click.echo(msg)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ecsmgmt', 'ecsmgmt._util', 'ecsmgmt.secret-key', 'ecsmgmt.user']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'click>=8.0.1,<9.0.0',
 'inquirer>=2.7.0,<3.0.0',
 'python-ecsclient>=1.1.11,<2.0.0',
 'tabulate>=0.8.9,<0.9.0']

entry_points = \
{'console_scripts': ['ecsmgmt = ecsmgmt:cli']}

setup_kwargs = {
    'name': 'ecsmgmt-cli',
    'version': '0.1.0',
    'description': 'Small CLI tool for interacting with the ECS Management API',
    'long_description': None,
    'author': 'Dominik Rimpf',
    'author_email': 'dominik.rimpf@kit.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os.path

import click
from ecsclient.client import Client

from ._util.config import load_config
from ._util.const import CONTEXT_SETTINGS
from ._util.core import DynamicMultiCommandFactory

DynamicMultiCommand = DynamicMultiCommandFactory().create(__file__, __package__)


@click.command(cls=DynamicMultiCommand, context_settings=CONTEXT_SETTINGS)
@click.option(
    '-c', '--config', 'config_path',
    type=click.Path(dir_okay=False),
    default=os.path.join(click.get_app_dir('ecsmgmt-cli'), 'config.yml'),
    help='Configuration file in YAML format',
    callback=load_config,
    is_eager=True,
    expose_value=False,
    show_default=True,
)
@click.option(
    '-u', '--username',
    type=click.STRING,
    prompt=True,
    show_default=True,
)
@click.option(
    '-p', '--password',
    type=click.STRING,
    prompt=True,
    hide_input=True,
)
@click.option(
    '-e', '--endpoint', 'ecs_endpoint',
    type=click.STRING,
    help='ECS Management API Endpoint URI',
    show_default=True,
)
@click.pass_obj
def cli(obj, username, password, ecs_endpoint):
    """Small CLI Client for the ECS Management API.
    """
    client = Client(
        version='3',
        username=username,
        password=password,
        ecs_endpoint=ecs_endpoint,
        token_endpoint=f'{ecs_endpoint}/login',
        cache_token=False,
    )
    obj['client'] = client



import datetime

import click

from .._util.format import success_echo, pretty_table


@click.command()
@click.argument('user-id')
@click.option('-n', '--namespace', type=click.STRING, show_default=True)
@click.pass_obj
def cli(obj, user_id, namespace):
    """Get a users secret key
    """
    client = obj['client']

    res = client.secret_key.get(user_id=user_id, namespace=namespace)
    data = []
    for i in [1, 2]:
        if res[f'secret_key_{i}_exist']:
            if res[f'key_expiry_timestamp_{i}'] != '':
                timestamp = datetime.datetime.fromisoformat(res[f'key_expiry_timestamp_{i}'] + '+00:00')
                local_timestamp = timestamp.astimezone().strftime('%Y-%m-%d %H:%M:%S %Z')
            else:
                local_timestamp = 'None'
            data.extend([
                (f'({i}) Secret Key', res[f'secret_key_{i}']),
                (f'({i}) Expiry Timestamp', local_timestamp)
            ])
    msg = pretty_table(data, tablefmt='plain')
    if data:
        success_echo(f'Secret keys for user "{user_id}" in namespace "{namespace}":\n{msg}')
    else:
        click.echo(f'User "{user_id}" in namespace "{namespace}" has no secret keys.')
